@@ -27,13 +27,9 @@ vcgCurve <- function(mesh)
     {
         if (!inherits(mesh,"mesh3d"))
             stop("argument 'mesh' needs to be object of class 'mesh3d'")
+        mesh <- meshintegrity(mesh,facecheck=TRUE)
         vb <- mesh$vb[1:3,,drop=FALSE]
-        it <- mesh$it - 1
-        if (!is.matrix(vb))
-            stop("mesh has no vertices")
-        if (!is.matrix(it))
-            stop("mesh has no faces")
-        
+        it <- mesh$it - 1        
         dimit <- dim(it)[2]
         dimvb <- dim(vb)[2]
         storage.mode(it) <- "integer"
