@@ -1,6 +1,6 @@
 #include "typedef.h"
 #include "RvcgIO.h"
-#include <Rcpp.h>
+#include <RcppArmadillo.h>
 
 using namespace vcg;
 using namespace tri;
