@@ -5,7 +5,7 @@
 // to work with R
 #include "typedefTopo.h"
 #include "RvcgIO.h"
-#include <Rcpp.h>
+#include <RcppArmadillo.h>
 using namespace vcg;
 using namespace tri;
 using namespace Rcpp;
@@ -53,18 +53,18 @@ RcppExport SEXP Risolated(SEXP vb_ , SEXP it_, SEXP diam_, SEXP facenum_,SEXP si
 	chunks.push_back(diag);
 	chunkface.push_back(CCV[i1].first);
       }
+      
       if (!split) {
-      if (diameter == 0)
-	diameter = *std::max_element(chunks.begin(),chunks.end());
-  
-      if (connect < 0) {
-	delInfo= tri::Clean<TopoMyMesh>::RemoveSmallConnectedComponentsDiameter(m,diameter);
-      } else {
-	if (connect == 0)
-	  connect = *std::max_element(chunkface.begin(),chunkface.end());
-	delInfo = tri::Clean<TopoMyMesh>::RemoveSmallConnectedComponentsSize(m,connect);
-      }
-      if (!silent)
+	if (diameter == 0)
+	  diameter = *std::max_element(chunks.begin(),chunks.end());
+	if (connect < 0) {
+	  delInfo= tri::Clean<TopoMyMesh>::RemoveSmallConnectedComponentsDiameter(m,diameter);
+	} else {
+	  if (connect == 0)
+	    connect = *std::max_element(chunkface.begin(),chunkface.end());
+	  delInfo = tri::Clean<TopoMyMesh>::RemoveSmallConnectedComponentsSize(m,connect);
+	}
+	if (!silent)
 	Rprintf("Removed %i connected components out of %i\n", delInfo.second, delInfo.first); 
       tri::Clean<TopoMyMesh>::RemoveUnreferencedVertex(m);
       // get a vector of which vertices were removed
@@ -90,22 +90,26 @@ RcppExport SEXP Risolated(SEXP vb_ , SEXP it_, SEXP diam_, SEXP facenum_,SEXP si
 	
       } else {
 	int length = CCV.size();
-	List out(length);
+	std::list<List> out;
 	for(unsigned int i=0; i < CCV.size(); ++i ) {
 	  TopoMyMesh destMesh;
-	  tri::UpdateSelection<TopoMyMesh>::FaceClear(m);
+	  tri::UpdateSelection<TopoMyMesh>::Clear(m);
 	  CCV[i].second->SetS();
-	  tri::UpdateSelection<TopoMyMesh>::FaceConnectedFF(m);
-	  tri::UpdateSelection<TopoMyMesh>::VertexClear(m);
-	  tri::UpdateSelection<TopoMyMesh>::VertexFromFaceLoose(m);
-
-	  tri::Append<TopoMyMesh,TopoMyMesh>::Mesh(destMesh, m, true);
-	  //tri::UpdateBounding<TopoMyMesh>::Box(destMesh);						// updates bounding box
-	  tri::UpdateNormal<TopoMyMesh>::PerVertexNormalized(destMesh);				// vertex normals
+	  tri::UpdateSelection<TopoMyMesh>::FaceConnectedFF(m,true);
+	  //tri::UpdateSelection<TopoMyMesh>::Clear(m);
+	  tri::UpdateSelection<TopoMyMesh>::VertexFromFaceLoose(m,true);
 	  
-	  out[i] = Rvcg::IOMesh<TopoMyMesh>::RvcgToR(destMesh,true);
+	  tri::Append<TopoMyMesh,TopoMyMesh>::Mesh(destMesh, m, true);
+	  unsigned int facenum = destMesh.fn;
+	  tri::UpdateBounding<TopoMyMesh>::Box(destMesh);
+	  float mybboxdiam = destMesh.bbox.Diag();
+	  if ((connect < 0 && mybboxdiam >= diameter) || (connect >= 0 && facenum >= connect))
+	    {
+	      tri::UpdateNormal<TopoMyMesh>::PerVertexNormalized(destMesh);				// vertex normals
+	      out.push_back(Rvcg::IOMesh<TopoMyMesh>::RvcgToR(destMesh,true));
+	    }
 	}
-	return out;
+	return wrap(out);
       }
 	
     } 
