@@ -53,14 +53,15 @@ class GetOpt {
     QString description;
     QVariant *value;
     QString *string_value;
+	float *float_value;
     double *double_value;
     int *int_value;
     bool *boolean_value;
 
-    Option(): value(NULL), string_value(NULL), double_value(NULL), int_value(NULL), boolean_value(NULL) {}
+	Option(): value(NULL), string_value(NULL), float_value(NULL), double_value(NULL), int_value(NULL), boolean_value(NULL) {}
     Option(Type _type, char _o, QString _name, QString _descr):
         type(_type), o(_o), name(_name), description(_descr),
-        value(NULL), string_value(NULL), double_value(NULL), int_value(NULL), boolean_value(NULL) {}
+		value(NULL), string_value(NULL), float_value(NULL), double_value(NULL), int_value(NULL), boolean_value(NULL) {}
   };
 
   bool unlimitedArgs;
@@ -82,6 +83,7 @@ class GetOpt {
   //add a valued option (v will be left untouched if the option is not given)
   void addOption(char s, const QString &longname, const QString &description, QVariant *v);
   void addOption(char s, const QString &longname, const QString &description, QString *v);
+  void addOption(char s, const QString &longname, const QString &description, float *v);
   void addOption(char s, const QString &longname, const QString &description, double *v);
   void addOption(char s, const QString &longname, const QString &description, int *v);
   void addOption(char s, const QString &longname, const QString &description, bool *v);
@@ -90,6 +92,7 @@ class GetOpt {
   //add an argument
   void addArgument(const QString &name, const QString &description, QVariant *v);
   void addArgument(const QString &name, const QString &description, QString *v);
+  void addArgument(const QString &name, const QString &description, float *v);
   void addArgument(const QString &name, const QString &description, double *v);
   void addArgument(const QString &name, const QString &description, int *v);
   void addArgument(const QString &name, const QString &description, bool *v);
